<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use Teknisa\MAN\Services;
use Teknisa\MAN\Utilities;

class PrinterParams extends \Teknisa\MAN\Controllers\TeknisaCrud {
    
    const dataSourceAmbiente   = 'PrinterParamsAreas';
    const dataSourceTipoScreen = 'PrinterParamsConfScreen';
    const dataSourcePrinter    = 'PrinterParamsPrinters';
    protected $parameterBagColumnsAmbiente     = array('NRCONFTELA', 'CDFILIAL');
    protected $parameterBagColumnsTipoScreen   = array('NRORG', 'NRCONFTELA');
    protected $parameterBagColumnsPrinters     = array('NRORG', 'CDLOJA', 'CDFILIAL');
	protected $salesMessage;
	protected $produtoService;
    protected $printerParamsService;
    protected $util;
    protected $ocorrenciaService;	

	public function __construct(Utilities\SalesMessage $salesMessage, Services\Produto $produtoService, Services\PrinterParams $printerParamsService, Utilities\Util $util, Services\Ocorrencia $ocorrenciaService, $crudService, $parameterBag) {
		$this->salesMessage          = $salesMessage;
		$this->produtoService 	     = $produtoService;
        $this->printerParamsService  = $printerParamsService;
        $this->util                  = $util;     
        $this->ocorrenciaService     = $ocorrenciaService;
        parent::__construct($crudService, $parameterBag);
	}

    public function getProductProdLoja(Request\Filter $request, Response $response) {
        try {
	        $params = $request->getFilterCriteria()->getConditions();
            $params = $this->util->getParams($params);
            if($params['PARAMDATA'] == "Printer"){
                unset($params['PARAMDATA']);
                $printers      = $this->printerParamsService->getPrinters($params['CDFILIAL'], $params['CDLOJA'], $params['NRORG']);
                $response->addDataSet(new DataSet('PrinterParamsPrinters', $printers));
            } else{
        		$paramProducts = $this->produtoService->buscaProdutosProdLoja($params);
                $departments   = $this->printerParamsService->getDepartments($params['NRORG']);
                $productsAdd   = $this->printerParamsService->getProductsForAdd($params['CDFILIAL'], $params['CDLOJA'], $params['NRORG']);
        		
                $response->addDataSet(new DataSet('PrinterParamsProducts', $paramProducts));
                $response->addDataSet(new DataSet('PrinterParamsDepartment', $departments));
                $response->addDataSet(new DataSet('PrinterParamsProductsForAdd', $productsAdd));
            }
    		

    	} catch (\Exception $e) {
    		$response->addMessage(new Message($e->getMessage()));
    	}
    }
    
    public function findPrinter (DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourcePrinter;
        $this->parameterBagColumns = $this->parameterBagColumnsPrinters;
        parent::find($request,$response);
    }
    
    public function findArea(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceAmbiente;
        $this->parameterBagColumns = $this->parameterBagColumnsAmbiente;
        parent::find($request,$response);
    }

    public function getAreas (Request\Filter $request, Response $response) {
        $params = $request->getFilterCriteria()->getConditions();
        $searchValue = '%%';
        foreach($params as $p){
            if($p['columnName'] == 'CDAMBIENTE|NMAMBIENTE'){
                if($p['operator'] == 'MAPPED_LIKE_ALL')
                    $searchValue = $p['value'][0]['_ALL'];
                if($p['operator'] == 'LIKE_ALL' && $p['value'] != '%%')
                    $searchValue = $p['value'];
            }
            
        }
        $params = $this->util->getParams($params);
        $params['CDAMBIENTE|NMAMBIENTE'] = $searchValue;
        try {

            $areas = $this->printerParamsService->getAreas($params['CDFILIAL'], $params['NRCONFTELA'], $params['CDAMBIENTE|NMAMBIENTE']);
            $response->addDataSet(new DataSet('PrinterParamsAreas1', $areas));    

        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function getProdAreas (Request\Filter $request, Response $response) {
        $params = $request->getFilterCriteria()->getConditions();
        $params = $this->util->getParams($params);

        try {

            $prodAreas = $this->printerParamsService->getProdAreas(
                $params['CDFILIAL'],
                $params['CDLOJA'],
                $params['CDPRODUTO']
            );

            $response->addDataSet(new DataSet('PrinterParamsProdAreas', $prodAreas));    

        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }
    
    public function findConfScreen (DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceTipoScreen;
        $this->parameterBagColumns = $this->parameterBagColumnsPrinters;
        parent::find($request,$response);
    }

    public function getConfScreen (Request\Filter $request, Response $response) {
        $params = $request->getFilterCriteria()->getConditions();
        $params = $this->util->getParams($params);

        try {

            $confScreens = $this->printerParamsService->getConfScreen(
                $params['CDFILIAL'],
                $params['CDLOJA']
            );

            $response->addDataSet(new DataSet('PrinterParamsConfScreen', $confScreens));    

        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    private function isDuplicateRegister ($CDFILIAL, $CDLOJA, $CDPRODUTO, $NRCONFTELA, $CDAMBIENTE, $NRSEQIMPRLOJA, $DTINIVIGENCIA) {
        $registers = $this->printerParamsService->getProdAreaDuplicate(
            $CDFILIAL, 
            $CDLOJA, 
            $CDPRODUTO, 
            $NRCONFTELA, 
            $CDAMBIENTE, 
            $NRSEQIMPRLOJA,
            $DTINIVIGENCIA
        );

        return $registers;
    }
    
    private function isDuplicateRegisterEdit ($CDFILIAL, $CDLOJA, $CDPRODUTO, $NRCONFTELA, $CDAMBIENTE, $NRSEQIMPRLOJA, $NRPRODAMBIENTE) {
        $registers = $this->printerParamsService->getProdAreaDuplicateEdit(
            $CDFILIAL, 
            $CDLOJA, 
            $CDPRODUTO, 
            $NRCONFTELA, 
            $CDAMBIENTE, 
            $NRSEQIMPRLOJA,
            $NRPRODAMBIENTE
        );

        return $registers;
    }

    public function deleteProdArea (DTO\Request $request, DTO\Response $response) {
        $params = $request->getDataSet()->getRows();
        try {

            //$nrProdAreas = json_decode($params[0]['NRPRODAMBIENTE'], true);

            $this->printerParamsService->beginTransaction();

            foreach ($params[0]['NRPRODAMBIENTE'] as $NRPRODAMBIENTE) {
                $this->printerParamsService->deleteProdArea(
                    $params[0]['CDFILIAL'],
                    $params[0]['CDLOJA'],
                    $params[0]['CDPRODUTO'],
                    $NRPRODAMBIENTE
                );
            }

            $prodAreas = $this->printerParamsService->getProdAreas(
                $params[0]['CDFILIAL'],
                $params[0]['CDLOJA'],
                $params[0]['CDPRODUTO']
            );

            $this->printerParamsService->commit();

            $response->addNotification(new Notification($this->salesMessage->getMessage(53)), Notification::TYPE_SUCCESS);
            $response->addDataSet(new DataSet('PrinterParamsProdAreas', $prodAreas));

        } catch (\Exception $e) {
            $this->printerParamsService->rollBack();
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function saveProdArea (DTO\Request $request, DTO\Response $response) {
        $params = $request->getDataSet()->getRows();
        try {
            if($params[0]["__is_new"] == true){
                if (empty($this->isDuplicateRegister(
                    $params[0]['CDFILIAL'],
                    $params[0]['CDLOJA'],
                    $params[0]['CDPRODUTO'],
                    $params[0]['NRCONFTELA'],
                    $params[0]['CDAMBIENTE'],
                    $params[0]['NRSEQIMPRLOJA'],
                    $params[0]['DTINIVIGENCIA']
                    ))) {
    
                    $NRPRODAMBIENTE = $this->util->newCode('PRODAMBIENTE'.$params[0]['CDFILIAL'].$params[0]['CDLOJA'].$params[0]['CDPRODUTO'], 10);
    
                    $this->printerParamsService->insertProdArea(
                        $params[0]['CDFILIAL'],
                        $params[0]['CDLOJA'],
                        $params[0]['CDPRODUTO'],
                        $NRPRODAMBIENTE,
                        $params[0]['NRCONFTELA'],
                        $params[0]['CDAMBIENTE'],
                        $params[0]['NRSEQIMPRLOJA'],
                        $params[0]['DTINIVIGENCIA']
                    );
    
                    $prodAreas = $this->printerParamsService->getProdAreas(
                        $params[0]['CDFILIAL'],
                        $params[0]['CDLOJA'],
                        $params[0]['CDPRODUTO']
                    );
    
                    $response->addNotification(new Notification($this->salesMessage->getMessage(52)), Notification::TYPE_SUCCESS);
                    $response->addDataSet(new DataSet('PrinterParamsProdAreas', $prodAreas));
    
                } else {
                    throw new \Exception($this->salesMessage->getMessage(59));
                }
            }else{
                if (empty($this->isDuplicateRegisterEdit(
                    $params[0]['CDFILIAL'],
                    $params[0]['CDLOJA'],
                    $params[0]['CDPRODUTO'],
                    $params[0]['NRCONFTELA'],
                    $params[0]['CDAMBIENTE'],
                    $params[0]['NRSEQIMPRLOJA'],
                    $params[0]['NRPRODAMBIENTE']
                    ))) {
                $this->printerParamsService->updateProdArea(
                    $params[0]['NRCONFTELA'],
                    $params[0]['CDAMBIENTE'],
                    $params[0]['NRSEQIMPRLOJA'],
                    $params[0]['CDFILIAL'],
                    $params[0]['CDLOJA'],
                    $params[0]['CDPRODUTO'],
                    $params[0]['NRPRODAMBIENTE']
                );
    
                $prodAreas = $this->printerParamsService->getProdAreas(
                    $params[0]['CDFILIAL'],
                    $params[0]['CDLOJA'],
                    $params[0]['CDPRODUTO']
                );
    
                $response->addNotification(new Notification($this->salesMessage->getMessage(54)), Notification::TYPE_SUCCESS);
                $response->addDataSet(new DataSet('PrinterParamsProdAreas', $prodAreas));
                
                } else {
                    throw new \Exception($this->salesMessage->getMessage(59));
                }
            }
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }
	
	public function cadlojaExcluirImprossoraAmbiente(DTO\Request $request, DTO\Response $response){
	    $rows = $this->util->getParamsAsArray($request);
	    try{
	         $this->printerParamsService->cadlojaExcluirImprossoraAmbiente($rows);
	    }catch(\Exception $e){
	       throw $e;
	    }
	}
	
}