<?php

namespace Teknisa\MAN\Services;

use Teknisa\MAN\Utilities;

class Produto {
	
	protected $entityManager;

	public function __construct(\Doctrine\ORM\EntityManager $entityManager)  {
		$this->entityManager = $entityManager;
	}

	public function buscaProdutos() {
		return $this->entityManager->getConnection()->fetchAll(Utilities\Query::BUSCA_PRODUTOS);
	}
	
	public function buscaProdutoEspecifico($CDPRODUTO) {
		$params = array(
			':CDPRODUTO'   => $CDPRODUTO
		);
		return $this->entityManager->getConnection()->fetchAll(Utilities\Query::BUSCA_PRODUTO_ESPECIFICO, $params);
	}

	public function buscaProdutosCardapioEComposicao($NRCONFTELA, $CDFILIAL) {
		$params = array(
			':CDFILIAL'   => $CDFILIAL,
			':NRCONFTELA' => $NRCONFTELA
		);

		return $this->entityManager->getConnection()->fetchAll(Utilities\Query::GET_PRODUCTS_AND_COMPOSITIONS, $params);
	}

	public function buscaProdutosProdLoja ($row, $allFields = null, $nivelInicial = null) {
		$query = '';
		$params = array(
				':CDFILIAL'  => $row['CDFILIAL'],
				':CDLOJA'    => $row['CDLOJA'],
				':NRORG'     => $row['NRORG']
		);
		if($allFields !== null) {
			$params = array_merge($params,array(
				':ALLFIELDS' => $allFields
			));
			$query = self::GET_PRODUCTS_PRODLOJA_WITH_ALLFIELDS;
		} else if (isset($nivelInicial)) {
			if(strlen($nivelInicial) > 1){
					$nivelInicial = substr($nivelInicial, 1, -1);
					$params = array_merge($params,array(
					':NIVELINICIAL' => $nivelInicial.'%'
				));
			}
			$query = self::GET_PRODUCTS_PRODLOJA_WITH_NIVELINICIAL;
		} else {
			$query = self::GET_PRODUCTS_PRODLOJA;
		}
		
		return $this->entityManager->getConnection()->fetchAll($query, $params);	
		
	}

	public function buscaObservacaoProduto($CDPRODUTO) {
		$params = array(
			':CDPRODUTO'   => $CDPRODUTO
		);
		return $this->entityManager->getConnection()->fetchAll(Utilities\Query::BUSCA_OBSERVACAO_PRODUTO, $params);
	}

	public function buscaProdutosRange($CDARVPROD, $CDARVPROD2){
		$params = array(
			':CDARVPROD' => $CDARVPROD,
			':CDARVPROD2' => $CDARVPROD2
		);
		
		return $this->entityManager->getConnection()->fetchAll(Utilities\Query::BUSCA_PRODUTOS_RANGE, $params);
	}

	public function buscaCDARVPROD($CDPRODUTO){
		$params = array(
			':CDPRODUTO' => $CDPRODUTO,			
		);		
		return $this->entityManager->getConnection()->fetchAll(Utilities\Query::BUSCA_CDARVPROD, $params);
	}

	/* Acrecimo não está funcionando 
	public function getAcrescimosProduto(){
		return $this->entityManager->getConnection()->fetchAll(Utilities\Query::GET_ACRESCIMO_PRODUTO);
	}
	
	public function getProductAcrescimo($CDPRODUTO, $CDPRODUTO2){
		$params = array(
			':CDPRODUTO' => $CDPRODUTO,
			':CDPRODUTO2' => $CDPRODUTO2
		);		
		return $this->entityManager->getConnection()->fetchAll(Utilities\Query::GET_PRODACRESCIMO_BY_CODIGOS, $params);
	}

	public function insereAcrescimo($CDACRESCIMO, $CDPRODUTO){
		$params = array(
			':CDACRESCIMO' => $CDACRESCIMO,
			':CDPRODUTO'   => $CDPRODUTO
		);
		
		return $this->entityManager->getConnection()->executeQuery(Utilities\Query::INSERE_ACRESCIMO, $params);
	}

	public function relacionaProdAcrescimo($CDACRESCIMO, $CDPRODUTO){
		$params = array(
			':CDACRESCIMO' => $CDACRESCIMO,
			':CDPRODUTO'   => $CDPRODUTO
		);
		
		return $this->entityManager->getConnection()->executeQuery(Utilities\Query::RELACIONA_PRODACRESCIMO, $params);
	}
	desabilitado para evitar erros na tela do manage Criação do Leo*/
	
	const GET_PRODUCTS_PRODLOJA_WITH_NIVELINICIAL = "
SELECT PL.CDFILIAL,
  PL.CDLOJA,
  PT.CDARVPROD,
  PT.CDPRODUTO,
  PT.NMPRODUTO,
  PL.IDIMPCANITEM,
  CASE PL.IDIMPCANITEM
    WHEN 'S'
    THEN 'Sim'
    WHEN 'N'
    THEN 'Não'
  END AS DSIMPCANITEM,
  PL.NRSEQIMPRPROD,
  PL.NRSEQIMPRPROD
  || ' - '
  || ILP.NMIMPRLOJA AS LABELIMPRPROD,
  ILP.NMIMPRLOJA    AS NMIMPRPROD,
  PL.NRSEQIMPRPUXA,
  PL.NRSEQIMPRPUXA
  || ' - '
  || ILX.NMIMPRLOJA AS LABELIMPRPUXA,
  ILX.NMIMPRLOJA    AS NMIMPRPUXA,
  PL.CDSETOR,
  PL.CDSETOR
  || ' - '
  || ST.NMSETOR AS NMSETOR
FROM PRODLOJA PL
JOIN PRODUTO PT
ON PL.CDPRODUTO = PT.CDPRODUTO
AND PL.NRORG    = PT.NRORG
LEFT JOIN IMPRLOJA ILP
ON ILP.NRSEQIMPRLOJA = PL.NRSEQIMPRPROD
AND ILP.CDFILIAL     = PL.CDFILIAL
AND ILP.CDLOJA       = PL.CDLOJA
AND ILP.NRORG        = PL.NRORG
LEFT JOIN IMPRLOJA ILX
ON ILX.NRSEQIMPRLOJA = PL.NRSEQIMPRPUXA
AND ILX.CDFILIAL     = PL.CDFILIAL
AND ILX.CDLOJA       = PL.CDLOJA
AND ILX.NRORG        = PL.NRORG
LEFT JOIN SETOR ST
ON ST.CDSETOR     = PL.CDSETOR
AND ST.NRORG      = PL.NRORG
WHERE PL.CDFILIAL = :CDFILIAL
AND PL.CDLOJA     = :CDLOJA
AND PL.NRORG      = :NRORG
AND PT.CDARVPROD LIKE :NIVELINICIAL
ORDER BY CDARVPROD
";
const GET_PRODUCTS_PRODLOJA_WITH_ALLFIELDS = "
        SELECT PL.CDFILIAL, PL.CDLOJA, PT.CDARVPROD, PT.CDPRODUTO, PT.NMPRODUTO, PL.IDIMPCANITEM, CASE PL.IDIMPCANITEM
                                                             WHEN 'S' THEN 'Sim'
                                                             WHEN 'N' THEN 'Não'
                                                         END AS DSIMPCANITEM,
               PL.NRSEQIMPRPROD, PL.NRSEQIMPRPROD || ' - ' || ILP.NMIMPRLOJA AS LABELIMPRPROD, ILP.NMIMPRLOJA AS NMIMPRPROD,
               PL.NRSEQIMPRPUXA, PL.NRSEQIMPRPUXA || ' - ' || ILX.NMIMPRLOJA AS LABELIMPRPUXA, ILX.NMIMPRLOJA AS NMIMPRPUXA,
               PL.CDSETOR, PL.CDSETOR || ' - ' || ST.NMSETOR AS NMSETOR
         FROM PRODLOJA PL JOIN PRODUTO PT
                            ON PL.CDPRODUTO = PT.CDPRODUTO
                AND PL.NRORG = PT.NRORG

                          LEFT JOIN IMPRLOJA ILP
                            ON ILP.NRSEQIMPRLOJA = PL.NRSEQIMPRPROD
                           AND ILP.CDFILIAL = PL.CDFILIAL
                           AND ILP.CDLOJA = PL.CDLOJA
                           AND ILP.NRORG = PL.NRORG

                          LEFT JOIN IMPRLOJA ILX
                            ON ILX.NRSEQIMPRLOJA = PL.NRSEQIMPRPUXA
                           AND ILX.CDFILIAL = PL.CDFILIAL
                           AND ILX.CDLOJA = PL.CDLOJA
                           AND ILX.NRORG = PL.NRORG

                          LEFT JOIN SETOR ST
                            ON ST.CDSETOR = PL.CDSETOR
                           AND ST.NRORG = PL.NRORG
                            
        WHERE PL.CDFILIAL =  :CDFILIAL
          AND PL.CDLOJA   =  :CDLOJA
          AND PL.NRORG    =  :NRORG
          AND INSTR(  PT.CDARVPROD || 
                      PT.NMPRODUTO || 
                      PL.NRSEQIMPRPROD || ' - ' || ILP.NMIMPRLOJA ||
                      PL.NRSEQIMPRPUXA || ' - ' || ILX.NMIMPRLOJA ,
                :ALLFIELDS) > 0 
      ORDER BY CDARVPROD
    ";
	
	const GET_PRODUCTS_PRODLOJA = "
        SELECT PL.CDFILIAL, PL.CDLOJA, PT.CDARVPROD, PT.CDPRODUTO, PT.NMPRODUTO, PL.IDIMPCANITEM, CASE PL.IDIMPCANITEM
                                                             WHEN 'S' THEN 'Sim'
                                                             WHEN 'N' THEN 'Não'
                                                         END AS DSIMPCANITEM,
               PL.NRSEQIMPRPROD, PL.NRSEQIMPRPROD || ' - ' || ILP.NMIMPRLOJA AS LABELIMPRPROD, ILP.NMIMPRLOJA AS NMIMPRPROD,
               PL.NRSEQIMPRPUXA, PL.NRSEQIMPRPUXA || ' - ' || ILX.NMIMPRLOJA AS LABELIMPRPUXA, ILX.NMIMPRLOJA AS NMIMPRPUXA,
               PL.CDSETOR, PL.CDSETOR || ' - ' || ST.NMSETOR AS NMSETOR
         FROM PRODLOJA PL JOIN PRODUTO PT
                            ON PL.CDPRODUTO = PT.CDPRODUTO
                AND PL.NRORG = PT.NRORG

                          LEFT JOIN IMPRLOJA ILP
                            ON ILP.NRSEQIMPRLOJA = PL.NRSEQIMPRPROD
                           AND ILP.CDFILIAL = PL.CDFILIAL
                           AND ILP.CDLOJA = PL.CDLOJA
                           AND ILP.NRORG = PL.NRORG

                          LEFT JOIN IMPRLOJA ILX
                            ON ILX.NRSEQIMPRLOJA = PL.NRSEQIMPRPUXA
                           AND ILX.CDFILIAL = PL.CDFILIAL
                           AND ILX.CDLOJA = PL.CDLOJA
                           AND ILX.NRORG = PL.NRORG

                          LEFT JOIN SETOR ST
                            ON ST.CDSETOR = PL.CDSETOR
                           AND ST.NRORG = PL.NRORG
                            
        WHERE PL.CDFILIAL =  :CDFILIAL
          AND PL.CDLOJA   =  :CDLOJA
          AND PL.NRORG    =  :NRORG
      ORDER BY CDARVPROD
    ";
}